@@ -10,7 +10,6 @@
  * See the file COPYING in the package base directory for details
  **/
 
-#include "vt_unify.h"
 #include "vt_unify_events_stats.h"
 #include "vt_unify_handlers.h"
 #include "vt_unify_hooks.h"
@@ -103,66 +102,244 @@ EventsAndStatsC::cleanUp()
    char filename1[STRBUFSIZE];
    char filename2[STRBUFSIZE];
 
-   int streams_num = (int)MyStreamIds.size();
+   int begin;
+   int end;
+   int step;
    int i;
 
-   const OTF_FileType common_file_type =
-      m_scope == SCOPE_EVENTS ? OTF_FILETYPE_EVENT : OTF_FILETYPE_STATS;
+   // base file type
+   const OTF_FileType base_file_type =
+      ( m_scope == SCOPE_EVENTS ) ? OTF_FILETYPE_EVENT : OTF_FILETYPE_STATS;
+
+   // temporary output file prefix
+   const std::string tmp_out_file_prefix =
+      Params.out_file_prefix + TmpFileSuffix;
 
    // remove local event/stat. files, if necessary
    //
    if( Params.doclean )
    {
+      // get begin, end, and step of loop removing files
+      //
+      begin = 0;
+      end = (int)MyStreamIds.size();
+      step = 1;
+      if( UnifyControlS::is_iofsl() )
+      {
+         end = (int)UnifyControlS::iofsl_num_servers;
+#ifdef VT_MPI
+         begin = MyRank;
+         step = NumRanks;
+#endif // VT_MPI
+      }
+
 #if defined(HAVE_OMP) && HAVE_OMP
 #     pragma omp parallel for private(i, filename1)
 #endif // HAVE_OMP
-      for( i = 0; i < streams_num; i++ )
+      for( i = begin; i < end; i+=step )
       {
-         const uint32_t & streamid = MyStreamIds[i];
+         // (re)initialize file type
+         OTF_FileType file_type = base_file_type;
 
-         // try to remove file without compression suffix
-         OTF_getFilename( Params.in_file_prefix.c_str(), streamid,
-            common_file_type, STRBUFSIZE, filename1 );
-         if( remove( filename1 ) == 0 )
-            PVPrint( 3, " Removed %s\n", filename1 );
+         // iterate over compression types (compressed, uncompressed)
+         for( uint8_t j = 0; j < 2; j++ )
+         {
+            // set compression bits of file type
+            //
+            if( j == 0 )
+            {
+               file_type &= ~OTF_FILECOMPRESSION_COMPRESSED;
+               file_type |= OTF_FILECOMPRESSION_UNCOMPRESSED;
+            }
+            else // j == 1
+            {
+               file_type &= ~OTF_FILECOMPRESSION_UNCOMPRESSED;
+               file_type |= OTF_FILECOMPRESSION_COMPRESSED;
+            }
 
-         // try to remove file with compression suffix
-         OTF_getFilename( Params.in_file_prefix.c_str(), streamid,
-            common_file_type | OTF_FILECOMPRESSION_COMPRESSED, STRBUFSIZE,
-            filename1 );
-         if( remove( filename1 ) == 0 )
-            PVPrint( 3, " Removed %s\n", filename1 );
+            if( UnifyControlS::is_iofsl() )
+            {
+               // iterate over IOFSL file types (all, idx)
+               for( uint8_t k = 0; k < 2; k++ )
+               {
+                  // set IOFSL bits of file type
+                  //
+                  if( k == 0 )
+                  {
+                     file_type &= ~OTF_FILETYPE_IOFSL_IDX;
+                     file_type |= OTF_FILETYPE_IOFSL_ALL;
+                  }
+                  else // k == 1
+                  {
+                     file_type &= ~OTF_FILETYPE_IOFSL_ALL;
+                     file_type |= OTF_FILETYPE_IOFSL_IDX;
+                  }
+
+                  // get file name
+                  OTF_getFilename( Params.in_file_prefix.c_str(), i,
+                     file_type, STRBUFSIZE, filename1 );
+                  // try to remove file
+                  if( remove( filename1 ) == 0 )
+                     PVPrint( 3, " Removed %s\n", filename1 );
+               }
+            }
+            else
+            {
+               // get file name
+               OTF_getFilename( Params.in_file_prefix.c_str(), MyStreamIds[i],
+                  file_type, STRBUFSIZE, filename1 );
+               // try to remove file
+               if( remove( filename1 ) == 0 )
+                  PVPrint( 3, " Removed %s\n", filename1 );
+            }
+         }
+      }
+   }
+
+   // remove previous created event/stat. output files
+   //
+   {
+      // get begin, end, and step of loop removing files
+      //
+      begin = 0;
+      end = (int)MyStreamIds.size();
+      step = 1;
+#if defined(HAVE_IOFSL) && HAVE_IOFSL
+      if( Params.doiofsl() )
+      {
+         end = (int)Params.iofsl_num_servers;
+#ifdef VT_MPI
+         begin = MyRank;
+         step = NumRanks;
+#endif // VT_MPI
+      }
+#endif // HAVE_IOFSL
+
+#if defined(HAVE_OMP) && HAVE_OMP
+#     pragma omp parallel for private(i, filename1)
+#endif // HAVE_OMP
+      for( i = begin; i < end; i+=step )
+      {
+         // (re)initialize file type
+         OTF_FileType file_type = base_file_type;
+
+         // iterate over compression types (compressed, uncompressed)
+         for( uint8_t j = 0; j < 2; j++ )
+         {
+            // set compression bits of file type
+            //
+            if( j == 0 )
+            {
+               file_type &= ~OTF_FILECOMPRESSION_COMPRESSED;
+               file_type |= OTF_FILECOMPRESSION_UNCOMPRESSED;
+            }
+            else // j == 1
+            {
+               file_type &= ~OTF_FILECOMPRESSION_UNCOMPRESSED;
+               file_type |= OTF_FILECOMPRESSION_COMPRESSED;
+            }
+
+#if defined(HAVE_IOFSL) && HAVE_IOFSL
+            if( Params.doiofsl() )
+            {
+               // iterate over IOFSL file types (all, idx)
+               for( uint8_t k = 0; k < 2; k++ )
+               {
+                  // set IOFSL bits of file type
+                  //
+                  if( k == 0 )
+                  {
+                     file_type &= ~OTF_FILETYPE_IOFSL_IDX;
+                     file_type |= OTF_FILETYPE_IOFSL_ALL;
+                  }
+                  else // k == 1
+                  {
+                     file_type &= ~OTF_FILETYPE_IOFSL_ALL;
+                     file_type |= OTF_FILETYPE_IOFSL_IDX;
+                  }
+
+                  // get file name
+                  OTF_getFilename( Params.out_file_prefix.c_str(), i, file_type,
+                     STRBUFSIZE, filename1 );
+                  // try to remove file
+                  if( remove( filename1 ) == 0 )
+                     PVPrint( 3, " Removed %s\n", filename1 );
+               }
+            }
+            else
+#endif // HAVE_IOFSL
+            {
+               // get file name
+               OTF_getFilename( Params.out_file_prefix.c_str(), MyStreamIds[i],
+                  file_type, STRBUFSIZE, filename1 );
+               // try to remove file
+               if( remove( filename1 ) == 0 )
+                  PVPrint( 3, " Removed %s\n", filename1 );
+            }
+         }
       }
    }
 
    // rename temporary event/stat. output files
    //
-
-   // get temporary output file prefix
-   const std::string tmp_out_file_prefix =
-      Params.out_file_prefix + TmpFileSuffix;
-
-   // get output file type
-   const OTF_FileType out_file_type = common_file_type |
-      ( Params.docompress ? OTF_FILECOMPRESSION_COMPRESSED : 0 );
-
-#if defined(HAVE_OMP) && HAVE_OMP
-#  pragma omp parallel for private(i, filename1, filename2)
-#endif // HAVE_OMP
-   for( i = 0; i < streams_num; i++ )
    {
-      const uint32_t & streamid = MyStreamIds[i];
+#if defined(HAVE_OMP) && HAVE_OMP
+#     pragma omp parallel for private(i, filename1, filename2)
+#endif // HAVE_OMP
+      for( i = begin; i < end; i+=step )
+      {
+         // (re)initialize file type
+         OTF_FileType file_type =
+            base_file_type | ( Params.docompress ?
+               OTF_FILECOMPRESSION_COMPRESSED :
+               OTF_FILECOMPRESSION_UNCOMPRESSED );
 
-      // get temporary file name
-      OTF_getFilename( tmp_out_file_prefix.c_str(), streamid,
-                       out_file_type, STRBUFSIZE, filename1 );
-      // get new file name
-      OTF_getFilename( Params.out_file_prefix.c_str(), streamid,
-                       out_file_type, STRBUFSIZE, filename2 );
+#if defined(HAVE_IOFSL) && HAVE_IOFSL
+         if( Params.doiofsl() )
+         {
+            // iterate over IOFSL file types (all, idx)
+            for( uint8_t k = 0; k < 2; k++ )
+            {
+               // set IOFSL bits of file type
+               //
+               if( k == 0 )
+               {
+                  file_type &= ~OTF_FILETYPE_IOFSL_IDX;
+                  file_type |= OTF_FILETYPE_IOFSL_ALL;
+               }
+               else // k == 1
+               {
+                  file_type &= ~OTF_FILETYPE_IOFSL_ALL;
+                  file_type |= OTF_FILETYPE_IOFSL_IDX;
+               }
 
-      // rename file
-      if( rename( filename1, filename2 ) == 0 )
-         PVPrint( 3, " Renamed %s to %s\n", filename1, filename2 );
+               // get old and new file name
+               //
+               OTF_getFilename( tmp_out_file_prefix.c_str(), i, file_type,
+                  STRBUFSIZE, filename1 );
+               OTF_getFilename( Params.out_file_prefix.c_str(), i, file_type,
+                  STRBUFSIZE, filename2 );
+
+               // try to rename file
+               if( rename( filename1, filename2 ) == 0 )
+                  PVPrint( 3, " Renamed %s to %s\n", filename1, filename2 );
+            }
+         }
+         else
+#endif // HAVE_IOFSL
+         {
+            // get old and new file name
+            //
+            OTF_getFilename( tmp_out_file_prefix.c_str(), MyStreamIds[i],
+               file_type, STRBUFSIZE, filename1 );
+            OTF_getFilename( Params.out_file_prefix.c_str(), MyStreamIds[i],
+               file_type, STRBUFSIZE, filename2 );
+
+            // try to rename file
+            if( rename( filename1, filename2 ) == 0 )
+               PVPrint( 3, " Renamed %s to %s\n", filename1, filename2 );
+         }
+      }
    }
 
    return !error;
@@ -187,21 +364,40 @@ EventsAndStatsC::rewrite()
    int i;
 
 #if defined(HAVE_OMP) && HAVE_OMP
-#  pragma omp parallel for private(i) schedule(static)
+#  pragma omp parallel for private(i) shared(error)
 #endif // HAVE_OMP
    for( i = 0; i < streams_num; i++ )
    {
+#if defined(HAVE_OMP) && HAVE_OMP
+#     pragma omp flush(error)
+      if( error ) continue;
+#else // HAVE_OMP
+      if( error ) break;
+#endif // HAVE_OMP
+
       const uint32_t & streamid = MyStreamIds[i];
 
-      // open file manager
+      // open file manager for reading
       //
-      OTF_FileManager * manager = OTF_FileManager_open( 1 );
-      assert( manager );
+      OTF_FileManager * rmanager = OTF_FileManager_open( 1 );
+      assert( rmanager );
+
+      // initialize IOFSL stuff for reading, if necessary
+      //
+      if( UnifyControlS::is_iofsl() )
+      {
+         OTF_IofslMode otf_iofsl_mode =
+            ( UnifyControlS::iofsl_mode == VT_IOFSL_MODE_MULTIFILE ) ?
+               OTF_IOFSL_MULTIFILE : OTF_IOFSL_MULTIFILE_SPLIT;
+
+         OTF_FileManager_setIofsl( rmanager, UnifyControlS::iofsl_num_servers,
+            0, otf_iofsl_mode, 0, 0, VT_TRACEID_BITMASK );
+      }
 
       // open stream for reading
       //
       OTF_RStream * rstream =
-      OTF_RStream_open( in_file_prefix.c_str(), streamid, manager );
+      OTF_RStream_open( in_file_prefix.c_str(), streamid, rmanager );
       assert( rstream );
 
       PVPrint( 3, " Opened OTF reader stream [namestub %s id %x]\n",
@@ -236,26 +432,54 @@ EventsAndStatsC::rewrite()
             OTF_RStream_closeStatsBuffer( rstream );
          }
 
+         // open file manager for writing
+         //
+         OTF_FileManager * wmanager =
+            OTF_FileManager_open( 2 /* 1xevent + 1xsnaps */ );
+         assert( wmanager );
+
+#if defined(HAVE_IOFSL) && HAVE_IOFSL
+         // initialize IOFSL stuff for writing, if necessary
+         //
+         if( Params.doiofsl() )
+         {
+            OTF_IofslMode otf_iofsl_mode =
+               ( Params.iofsl_mode == VT_IOFSL_MODE_MULTIFILE ) ?
+                  OTF_IOFSL_MULTIFILE : OTF_IOFSL_MULTIFILE_SPLIT;
+
+            uint32_t otf_iofsl_flags = 0;
+            if( ( Params.iofsl_flags & VT_IOFSL_FLAG_ASYNC_IO ) != 0 )
+               otf_iofsl_flags |= OTF_IOFSL_FLAG_NONBLOCKING;
+
+            OTF_FileManager_setIofsl( wmanager, Params.iofsl_num_servers,
+               Params.iofsl_servers, otf_iofsl_mode, otf_iofsl_flags, 0,
+               VT_TRACEID_BITMASK );
+         }
+#endif // HAVE_IOFSL
+
          // open stream for writing
          //
          OTF_WStream * wstream =
-         OTF_WStream_open( tmp_out_file_prefix.c_str(), streamid, manager );
+         OTF_WStream_open( tmp_out_file_prefix.c_str(), streamid, wmanager );
          assert( wstream );
 
          PVPrint( 3, " Opened OTF writer stream [namestub %s id %x]\n",
                   tmp_out_file_prefix.c_str(), streamid );
 
-#if (defined(VT_UNIFY_HOOKS_AEVENTS) || defined(VT_UNIFY_HOOKS_MARGINS))
+#if (defined(VT_UNIFY_HOOKS_AEVENTS) || defined(VT_UNIFY_HOOKS_MARGINS) || \
+     defined(VT_UNIFY_HOOKS_MSGMATCH_SNAPS))
          if( m_scope == SCOPE_EVENTS )
          {
-            // trigger generic hooks for opened event stream
+            // trigger generic hooks for opened event writer stream
             theHooks->triggerGenericHook(
-               VT_UNIFY_HOOKS_AEVENTS_GENID__EVENT_STREAM_OPEN |
-               VT_UNIFY_HOOKS_MARGINS_GENID__EVENT_STREAM_OPEN, 3,
+               VT_UNIFY_HOOKS_AEVENTS_GENID__EVENT_WSTREAM_OPEN |
+               VT_UNIFY_HOOKS_MARGINS_GENID__EVENT_WSTREAM_OPEN |
+               VT_UNIFY_HOOKS_MSGMATCH_SNAPS_GENID__EVENT_WSTREAM_OPEN, 3,
                &wstream, const_cast<uint32_t*>( &streamid ),
                const_cast<std::string*>( &in_file_prefix ) );
          }
-#endif // VT_UNIFY_HOOKS_AEVENTS || VT_UNIFY_HOOKS_MARGINS
+#endif // VT_UNIFY_HOOKS_AEVENTS || VT_UNIFY_HOOKS_MARGINS ||
+       // VT_UNIFY_HOOKS_MSGMATCH_SNAPS
 
          // set file compression
          //
@@ -439,19 +663,24 @@ EventsAndStatsC::rewrite()
             }
          }
 
-#if (defined(VT_UNIFY_HOOKS_AEVENTS) || defined(VT_UNIFY_HOOKS_MARGINS))
-         if( m_scope == SCOPE_EVENTS )
+#if (defined(VT_UNIFY_HOOKS_AEVENTS) || defined(VT_UNIFY_HOOKS_MARGINS) || \
+     defined(VT_UNIFY_HOOKS_MSGMATCH_SNAPS))
+         if( !error && m_scope == SCOPE_EVENTS )
          {
-            // trigger generic hooks for closing event stream
+            // trigger generic hooks for closing event writer stream
             theHooks->triggerGenericHook(
-               VT_UNIFY_HOOKS_AEVENTS_GENID__EVENT_STREAM_CLOSE |
-               VT_UNIFY_HOOKS_MARGINS_GENID__EVENT_STREAM_CLOSE, 1,
-               const_cast<uint32_t*>( &streamid ) );
+               VT_UNIFY_HOOKS_AEVENTS_GENID__EVENT_WSTREAM_CLOSE |
+               VT_UNIFY_HOOKS_MARGINS_GENID__EVENT_WSTREAM_CLOSE |
+               VT_UNIFY_HOOKS_MSGMATCH_SNAPS_GENID__EVENT_WSTREAM_CLOSE, 2,
+               &wstream, const_cast<uint32_t*>( &streamid ) );
          }
-#endif // VT_UNIFY_HOOKS_AEVENTS || VT_UNIFY_HOOKS_MARGINS
+#endif // VT_UNIFY_HOOKS_AEVENTS || VT_UNIFY_HOOKS_MARGINS ||
+       // VT_UNIFY_HOOKS_MSGMATCH_SNAPS
 
          // close writer stream
          OTF_WStream_close( wstream );
+         // close file manager for writing
+         OTF_FileManager_close( wmanager );
 
          PVPrint( 3, " Closed OTF writer stream [namestub %s id %x]\n",
                tmp_out_file_prefix.c_str(), streamid );
@@ -463,8 +692,8 @@ EventsAndStatsC::rewrite()
 
       // close reader stream
       OTF_RStream_close( rstream );
-      // close file manager
-      OTF_FileManager_close( manager );
+      // close file manager for reading
+      OTF_FileManager_close( rmanager );
 
       PVPrint( 3, " Closed OTF reader stream [namestub %s id %x]\n",
                in_file_prefix.c_str(), streamid );
