@@ -10,7 +10,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2008 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006-2009 Mellanox Technologies. All rights reserved.
  * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
  *                         reserved.
@@ -52,6 +52,7 @@
 BEGIN_C_DECLS
 
 #define HAVE_XRC (1 == OMPI_HAVE_CONNECTX_XRC)
+#define ENABLE_DYNAMIC_SL (1 == OMPI_ENABLE_DYNAMIC_SL)
 
 #define MCA_BTL_IB_LEAVE_PINNED 1
 #define IB_DEFAULT_GID_PREFIX 0xfe80000000000000ll
@@ -214,6 +215,9 @@ struct mca_btl_openib_component_t {
     uint32_t ib_rnr_retry;
     uint32_t ib_max_rdma_dst_ops;
     uint32_t ib_service_level;
+#if (ENABLE_DYNAMIC_SL)
+    uint32_t ib_path_record_service_level;
+#endif
     int32_t use_eager_rdma;
     int32_t eager_rdma_threshold; /**< After this number of msg, use RDMA for short messages, always */
     int32_t eager_rdma_num;
@@ -285,11 +289,18 @@ struct mca_btl_openib_component_t {
     ompi_free_list_t send_free_coalesced;
     /** Default receive queues */
     char* default_recv_qps;
+    /** GID index to use */
+    int gid_index;
     /** Whether we want a dynamically resizing srq, enabled by default */
     bool enable_srq_resize;
 #if BTL_OPENIB_FAILOVER_ENABLED
     int verbose_failover;
 #endif
+#if BTL_OPENIB_MALLOC_HOOKS_ENABLED
+    int use_memalign;
+    size_t memalign_threshold;
+    void* (*previous_malloc_hook)(size_t __size, const void*);
+#endif
 }; typedef struct mca_btl_openib_component_t mca_btl_openib_component_t;
 
 OMPI_MODULE_DECLSPEC extern mca_btl_openib_component_t mca_btl_openib_component;
@@ -378,6 +389,8 @@ typedef struct mca_btl_openib_device_t {
     mca_btl_openib_device_qp_t *qps;
     /* Maximum value supported by this device for max_inline_data */
     uint32_t max_inline_data;
+    /* Registration limit and current count */
+    uint64_t mem_reg_max, mem_reg_active;
 } mca_btl_openib_device_t;
 OBJ_CLASS_DECLARATION(mca_btl_openib_device_t);
 
@@ -455,6 +468,8 @@ struct mca_btl_openib_module_t {
     mca_btl_base_module_error_cb_fn_t error_cb; /**< error handler */
 
     mca_btl_openib_module_qp_t * qps;
+
+    int local_procs;                   /** number of local procs */
 };
 typedef struct mca_btl_openib_module_t mca_btl_openib_module_t;
 
